//
//  DelayViewController.m
//  hw1
//
//  Created by Uri Nieto on 1/11/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "DelayViewController.h"
#import "mo_audio.h"
#import "mo_accel.h"
#import "mo_touch.h"
#import "voice.h"

#define SRATE 44100
#define FRAMESIZE 128
#define NUMCHANNELS 2
#define PINCHFACTOR 100
#define INITIALDELAYLEN 10000
#define INITIALDELAYFACTOR 0.5
#define MINKNOT -150.0
#define MAXKNOT 150.0

double g_fc = 440.0;
double g_fc_target = 440.0;
double g_fc_slew = 0.01;

double g_rotation = 1.0;
double g_initial_rotation = 0.0;
double g_final_rotation = 0.0;

// delay
Voice *g_voice;
SAMPLE g_delay_len = INITIALDELAYLEN;   //Delay Length: Default 5000 frames
SAMPLE g_delay_factor = INITIALDELAYFACTOR; //Decay factor of our delay. Default 0.95

// pinch
CGFloat g_initial_distance = 0;
CGFloat g_final_distance = 0;


// implement audio callback here
void audioCallback( SAMPLE * buffer, UInt32 frameSize, void * userData) {
  
  // Synthesize the delay
  g_voice->synthesize(buffer, buffer, frameSize);
    
}

void accelCallback( double x, double y, double z, void * data) {
  //NSLog([NSString stringWithFormat: @"x: %f, y: %f, z: %f", x,y,z]);
  
  g_fc = 440 + 50.0*x;
  
}

CGFloat distanceBetweenTwoPoints( CGPoint fromPoint, CGPoint toPoint) {
  
  float x = toPoint.x - fromPoint.x;
  float y = toPoint.y - fromPoint.y;
  return sqrt(x * x + y * y);
  
}

void touchCallback( NSSet * touches, UIView *view, const std::map<int, 
                   MoTouchTrack> & touchPts,
                   void * data  ) {
  
  long count = [touches count];
  
  //NSLog(@"View Tag: %d", [view tag]);
  
  // If touching the "Pinch" zone:
  if ([view tag] == 2) {
    
    if (count == 2) {
      // Possible Pinch
      
      // Get the two touches
      UITouch *touch1 = [[touches allObjects] objectAtIndex:0];
      UITouch *touch2 = [[touches allObjects] objectAtIndex:1];
      
      // If some of these began,calculate the initial distance
      if ([touch1 phase] == UITouchPhaseBegan || [touch2 phase] == UITouchPhaseBegan) {
        g_initial_distance = distanceBetweenTwoPoints([touch1 locationInView:view], [touch2 locationInView:view]);
      }
      // If some of these moved, then we already have the initial distance, let's calculate the final
      else if ([touch1 phase] == UITouchPhaseMoved || [touch2 phase] == UITouchPhaseMoved) {
        g_final_distance = distanceBetweenTwoPoints([touch1 locationInView:view], [touch2 locationInView:view]);
        
        // Pinch out
        if(g_initial_distance > g_final_distance) {
          NSLog(@"Pinch Out");
          
          // Decrease the Delay Length
          g_delay_len -= PINCHFACTOR;
          
          if (g_delay_len < 0){
            g_delay_len = 0;
          }
          g_voice->setDelayLen(g_delay_len);
          g_voice->initDelayBuffer();
        }
        // Pinch in
        else {
          NSLog(@"Pinch In");
    
          // Increase the Delay Length
          g_delay_len += PINCHFACTOR;
          
          if (g_delay_len > MAX_DELAY){
            g_delay_len = MAX_DELAY;
          }
          g_voice->setDelayLen(g_delay_len);
          g_voice->initDelayBuffer();
        }
        
        // update initial distance
        g_initial_distance = g_final_distance;
        
        // update label
        UILabel *delayLenLabel = (UILabel*)[[view superview] viewWithTag:1];
        [delayLenLabel setText: [NSString stringWithFormat:@"%.0f samples", g_delay_len]];
      }
    }
  } // End of "Pinch" Zone
  
  // If touching the Knot
  else if ([view tag] == 3) {
    
    // Only one touch
    if (count == 1) {
      
      // Get the touch
      UITouch *touch = [[touches allObjects] objectAtIndex:0];
      if ([touch phase] == UITouchPhaseBegan) {
        g_initial_rotation = [touch locationInView: view].x;
      }
      else if ([touch phase] == UITouchPhaseMoved) {
        
        CGAffineTransform rotate = CGAffineTransformMakeRotation( g_rotation / 180.0 * MY_PIE );
        [[view viewWithTag:4] setTransform: rotate];
        
        g_final_rotation = [touch locationInView: view].x;
        
        NSLog(@"Initial: %.1f, Final: %.1f", g_initial_rotation, g_final_rotation);
        
        if (g_initial_rotation < g_final_rotation) {
          g_rotation += 6.0;
        }
        
        else if (g_initial_rotation > g_final_rotation) {
          g_rotation -= 6.0;
        }
        
        // Check boundaries
        if (g_rotation < MINKNOT) {
          g_rotation = MINKNOT;
        }
        else if (g_rotation > MAXKNOT) {
          g_rotation = MAXKNOT;
        }

        // Update initial pos
        g_initial_rotation = g_final_rotation;
        
        // Set Delay Factor:
        g_delay_factor = (1/(MAXKNOT-MINKNOT))*g_rotation + (1.0/2.0);
        
        // Set the value for the Delayed Voice
        g_voice->setDelayFactor(g_delay_factor);
        
        // Update the label
        [(UILabel*)[[view superview] viewWithTag:8] setText: [NSString stringWithFormat:@"%.1f", g_delay_factor]];
      }
    }
  }
}

@implementation DelayViewController

@synthesize delayLenLabel;
@synthesize delayFactorLabel;
@synthesize knotImageView;

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
  [super viewDidLoad];
  
  // Init Delay Voice
  g_voice = new Voice(g_delay_len, g_delay_factor, false);
  g_voice->initDelayBuffer();
  
  // Init Audio
  bool result = MoAudio::init( SRATE, FRAMESIZE, NUMCHANNELS );
  
  if (!result) {
    NSLog(@"cannot initialize real-time audio!");
    return;
  }
  
  // start audio layer
  result = MoAudio::start( audioCallback, NULL);
  
  if (!result) {
    NSLog(@"cannot start real-time audio!");
    return;
  }
  
  // Init Accel
  MoAccel::instance()->addCallback( accelCallback, NULL );
  
  // Init Touch
  MoTouch::instance()->addCallback( touchCallback, NULL );
  
  // Set Labels Text
  [delayLenLabel setText: [NSString stringWithFormat:@"%.0f samples", g_delay_len]];
  [delayFactorLabel setText: [NSString stringWithFormat:@"%.1f", g_delay_factor]];
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

- (IBAction)delayFactorChanged:(id)sender {
  UISlider *slider = (UISlider*)sender;
  
  // Get the new value
  g_delay_factor = [slider value];
  
  // Set the value for the Delayed Voice
  g_voice->setDelayFactor(g_delay_factor);
  
  // Update the label
  [delayFactorLabel setText: [NSString stringWithFormat:@"%.1f", g_delay_factor]];
}


@end
