//
//  FMViewController.m
//  hw1
//
//  Created by Uri Nieto on 1/10/10.
//  Copyright 2010 Stanford University. All rights reserved.
//

#import "FMViewController.h"
#import "mo_audio.h"
#import "mo_accel.h"
#import "mo_touch.h"

#define SRATE           44100
#define FRAMESIZE       128
#define NUMCHANNELS     2
#define MODULATEDFREQ   440.0
#define CARRIERFREQ     100.0
#define MODULATIONINDEX 1.0

// The accelerometer and the sliders will change these:
double g_fm = MODULATEDFREQ;
double g_fc = CARRIERFREQ;

// The sliders will change these:
double g_init_fm = g_fm;
double g_init_fc = g_fc;

double g_mod_index = MODULATIONINDEX;

bool g_touching = false;
bool g_manual = false;

// implement audio callback here
void audioCallback( Float32 * buffer, UInt32 frameSize, void * userData) {
  
  static float phase_c = g_fc / SRATE;
  static float phase_m = g_fm / SRATE;
  
  for (int i=0; i<frameSize; i++) {
    
    // If we're not in the manual mode or we are touching the screen, let's play the FM
    if (!g_manual || g_touching) {
      buffer[2*i] = buffer[2*i + 1] = sin(2.0*M_PI*phase_c + g_mod_index*cos(2.0*M_PI*phase_m));
    }
    else {
      buffer[2*i] = 0;
    }
    
    // Update our phases
    phase_c += g_fc/(SRATE*1.0);
    phase_m += g_fm/(SRATE*1.0);
    
    if (phase_c > 1.0f)
      phase_c -= 1.0f;
    if (phase_m > 1.0f)
      phase_m -= 1.0f;
  }
  
}

void accelCallback( double x, double y, double z, void * data) {
  //NSLog([NSString stringWithFormat: @"x: %f, y: %f, z: %f", x,y,z]);
  
  g_fm = g_init_fm + 50.0*y;
  
  g_fc = g_init_fc + 50.0*x;
  
}


@implementation FMViewController

@synthesize modulatedFreqLabel;
@synthesize carrierFreqLabel;
@synthesize mIndexLabel;

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
  [super viewDidLoad];
  
  // Init Trigger Note Button
  [self initTriggerNoteButton];
  
  // Init Audio
  bool result = MoAudio::init( SRATE, FRAMESIZE, NUMCHANNELS );
  
  if (!result) {
    NSLog(@"cannot initialize real-time audio!");
    return;
  }
  
  // start audio layer
  result = MoAudio::start( audioCallback, NULL);
  
  if (!result) {
    NSLog(@"cannot start real-time audio!");
    return;
  }
  
  // Init Accel
  MoAccel::instance()->addCallback( accelCallback, NULL );
  
  // Set Labels Text
  [modulatedFreqLabel setText: [NSString stringWithFormat:@"%.0f Hz", g_init_fm]];
  [carrierFreqLabel setText: [NSString stringWithFormat:@"%.0f Hz", g_init_fc]];
  [mIndexLabel setText: [NSString stringWithFormat:@"%.0f", g_mod_index]];
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
  [super dealloc];
}

- (void)initTriggerNoteButton {
  // Create the Button
  triggerNoteButton = [UIButton buttonWithType: UIButtonTypeRoundedRect];
  
  // Set the Frame (Position)
  [triggerNoteButton setFrame: CGRectMake(100.0, 560.0, 120.0, 60.0)];
  
  // Set the Target Actions
  [triggerNoteButton addTarget:self action:@selector(triggerNoteButtonPressed:) forControlEvents: UIControlEventTouchDown];
  [triggerNoteButton addTarget:self action:@selector(triggerNoteButtonUnpressed:) forControlEvents: UIControlEventTouchUpInside];
  
  // Set the Titles
  [triggerNoteButton setTitle: @"Trigger Note!" forState: UIControlStateNormal];
  
  // Add it to the view
  [self.view addSubview: triggerNoteButton];
  
  // Hide it by default
  //[triggerNoteButton setHidden: YES];
  [triggerNoteButton setAlpha: 0.0];
}

- (void)triggerNoteButtonPressed:(id)sender {
  NSLog(@"Pressed");
  g_touching = true;
}

- (void)triggerNoteButtonUnpressed:(id)sender {
  NSLog(@"Unpressed");
  g_touching = false;
}

- (IBAction)switchPressed:(id)sender {
  
  UISwitch *my_switch = (UISwitch*)sender;
  if ([my_switch isOn]) {
    NSLog(@"Manual Mode On");
    
    g_manual = true;
    
    // Unhide the button
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:0.3];
    [triggerNoteButton setAlpha: 1.0];
    [triggerNoteButton setFrame:CGRectMake(100.0, 360.0, 120.0, 60.0)];
    [UIView commitAnimations];
    
  }
  else {
    NSLog(@"Manual Mode Off");
    g_manual = false;
    
    // Hide the button
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:0.3];
    [triggerNoteButton setAlpha: 0.0];
    [triggerNoteButton setFrame:CGRectMake(100.0, 560.0, 120.0, 60.0)];
    [UIView commitAnimations];
  }
}

- (IBAction)changeModulatedFreq:(id)sender {
  UISlider *slider = (UISlider*)sender;
  
  // Get the new value
  g_init_fm = MODULATEDFREQ * 2 * [slider value];
  
  // Update the label
  [modulatedFreqLabel setText: [NSString stringWithFormat:@"%.0f Hz", g_init_fm]];
}

- (IBAction)changeCarrierFreq:(id)sender {
  UISlider *slider = (UISlider*)sender;
  
  // Get the new value
  g_init_fc = CARRIERFREQ * 2 * [slider value];
  
  // Update the label
  [carrierFreqLabel setText: [NSString stringWithFormat:@"%.0f Hz", g_init_fc]];
}

- (IBAction)changeModIndex:(id)sender {
  UISlider *slider = (UISlider*)sender;
  
  // Get the new value
  g_mod_index = [slider value];
  
  // Update the label
  [mIndexLabel setText: [NSString stringWithFormat:@"%.0f", g_mod_index]];
}


@end
