//
//  hw1ViewController.m
//  hw1
//
//  Created by Uri Nieto on 1/6/10.
//  Copyright Stanford University 2010. All rights reserved.
//

#import "hw1ViewController.h"
#import "mo_audio.h"
#import "mo_accel.h"
#import "mo_touch.h"

#define SRATE 44100
#define FRAMESIZE 128
#define NUMCHANNELS 2

double g_fc = 440.0;
double g_fOffset = 0.0;

// implement audio callback here
void audioCallback( Float32 * buffer, UInt32 frameSize, void * userData) {
  
  static float phase = g_fc / SRATE;
  
  for (int i=0; i<frameSize; i++) {
    buffer[2*i] = buffer[2*i + 1] = sin(2.0*M_PI*phase);
    phase += (g_fc + g_fOffset)/(SRATE*1.0);
    
    if (phase > 1.0f)
      phase -= 1.0f;
  }
  
}

void accelCallback( double x, double y, double z, void * data) {
  //NSLog([NSString stringWithFormat: @"x: %f, y: %f, z: %f", x,y,z]);
  
  g_fc = 440 + 50.0*x;
  
}

void touchCallback( NSSet * touches, UIView *view, const std::map<int, 
                   MoTouchTrack> & touchPts,
                   void * data  ) {
  NSLog(@"Touch!");
  
  long count = [touches count];
  
  if (count == 1) {
    for (UITouch *touch in touches) {
      CGPoint point = [touch locationInView:view];
      
      g_fOffset = 10*point.x;
    }
  }
}

@implementation hw1ViewController

@synthesize mySynthLabel;


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
  [super viewDidLoad];
  
  // Init Audio
  bool result = MoAudio::init( SRATE, FRAMESIZE, NUMCHANNELS );
  
  if (!result) {
    NSLog(@"cannot initialize real-time audio!");
    return;
  }
  
  // start audio layer
  result = MoAudio::start( audioCallback, NULL);
  
  if (!result) {
    NSLog(@"cannot start real-time audio!");
    return;
  }
  
  // Init Touch
  MoTouch::instance()->addCallback( touchCallback, NULL );
  
  // Init Accel
  MoAccel::instance()->addCallback( accelCallback, NULL );
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

- (IBAction)hitSinButton {
  NSLog(@"Hit Sine Button");
  mySynthLabel.text = @"Sine";
}

- (IBAction)hitFeedbackButton {
  NSLog(@"Hit Feedback Button");
  mySynthLabel.text = @"Feedback";
}

@end
