/******************************************/
/*
 voice.h
 by Uri Nieto, 2009
 
 Homework #2: Delay, Feedback, Moving Average, and Polyphony
 
 Music 256a
 Stanford University
 */
/******************************************/

#include "voice.h"
#include <iostream>
#include <cstdlib>
#include <math.h>
#include <string.h>

Voice::Voice(int delayLen, SAMPLE delayFactor, bool midi)  {
  m_writer = 0;           //Pointer to write to the delay buffer
  m_reader = 0;           //Pointer to read from the delay buffer
  m_delayFactor = delayFactor;   //Decay factor of our delay
  m_delayLen = delayLen;  //Lenght of our delay
  m_prev = 0.0;           //Empty first previous frame
  m_noise = false;        //No noise in our delay buffer by default
  m_midiInput = midi;     //Whether we are using midi input
  m_t = 0;                //Time
  m_pluck = 0;            //Pluck processed at the moment
}

//This function synthesize the delay and puts it into our
//output buffer obuffy.
//We have two indices in our delay buffer (m_reader and
//m_writer). At everyframe, the delay buffer at index
//m_reader will be read.
//At everyframe, wi will write into the delay buffer
//at index m_writer the content of our input buffer ibuffy
//plus the read delay and the previous frame times
//an atenuation factor called m_delayFactor
void Voice::synthesize(SAMPLE * obuffy, 
                       SAMPLE * ibuffy, 
                       long nBufferFrames ) {
  
  for (int i=0; i<nBufferFrames; i++) {
    //Read the input frame
    
    m_delay[2*m_writer] = ibuffy[2*i]*m_delayFactor;
    m_delay[2*m_writer + 1] = ibuffy[2*i + 1]*m_delayFactor;
        
    //Write the delay with moving average to the buffer
    m_delay[2*m_writer] += (m_delay[2*m_reader] + m_prev)*m_delayFactor;
    m_delay[2*m_writer + 1] += (m_delay[2*m_reader + 1] + m_prev)*m_delayFactor;
    
    //Update the prveious frame
    m_prev = m_delay[2*m_reader];
    m_prev = m_delay[2*m_reader + 1];
    
    //printf("m_writer: %d, m_reader: %d, m_delay[m_reader]: %f\n", m_writer, m_reader, m_delay[m_reader]);
    
    //Put the read frame of our buffer to the output
    obuffy[2*i] = m_delay[2*m_reader];
    obuffy[2*i + 1] = m_delay[2*m_reader + 1];
    
    //increment indices
    m_reader++;
    m_writer++;
    
    //wrap indices
    m_writer %= MAX_DELAY;
    m_reader %= MAX_DELAY;
    
    //Increment time
    m_t++;
  }
}

//Inits our Delay Buffer m_buffer
//First it zero outs the buffer
//Then it sets the indices and finally wraps the
//reader index in case it's out of bounds
void Voice::initDelayBuffer() {
  //Zero out our delay buffer:
  memset (m_delay, 0, MAX_DELAY*sizeof(SAMPLE));
  // set indices
  m_reader = m_writer - m_delayLen;
  // wrap
  if( m_reader < 0 ) m_reader += MAX_DELAY;
}

//Sets the m_noise var
void Voice::setNoise(bool noise) {
  
  //Sets our variable of noise
  m_noise = noise;
}

//Set Delay Length and set the indices correctly:
void Voice::setDelayLen(long delayLen) {
  m_delayLen = delayLen;
  // set indices
  m_reader = m_writer - m_delayLen;
  // wrap
  if( m_reader < 0 ) m_reader += MAX_DELAY;
}

void Voice::setDelayFactor(double delayFactor) {
  m_delayFactor = delayFactor;
}

void Voice::pluck(long delayLen)
{
  m_delayLen = delayLen;
  // set indices
  m_reader = m_writer - m_delayLen;
  // wrap
  if( m_reader < 0 ) m_reader += MAX_DELAY;
  m_pluck = m_delayLen;
  
  //Sets the delayFactor to 0.95 (in case we had set it down before)
  m_delayFactor = 0.95;
}

