//-----------------------------------------------------------------------------
// name: mo_accel.h
// desc: MoPhO API for accelerometer
//
// authors: Nick Bryan
//          Jieun Oh
//          Jorge Herrera
//          Ge Wang
//
//    date: Fall 2009
//    version: 0.1
//
// Stanford Mobile Phone Orchestra
//     http://mopho.stanford.edu/
//-----------------------------------------------------------------------------
#ifndef __MO_ACCEL_H__
#define __MO_ACCEL_H__

#include "mo_def.h"
#include <vector>


@interface AccelDelegate : NSObject <UIAccelerometerDelegate>
{ }
@end


// type definition for accelerometer callback function
typedef void (* MoAccelCallback)( double x, double y, double z, void * data );


//-----------------------------------------------------------------------------
// name: class MoAccel
// desc: accelerometer stuff (singleton)
//-----------------------------------------------------------------------------
class MoAccel
{
public: // setting values

    // this should be called on new values (UIAccelerometerDelegate)
    void update( double x, double y, double z );

    // set the global update interval
    void setUpdateInterval( double seconds );

    // get the global update interval
    double getUpdateInterval();

public: // getting values

    // returns current (x,y,z) values (for polling)
    void   getXYZ( double & px, double & py, double & pz ) const;

    // returns current (x) value (for polling)
    double getX() const;

    // returns current (y) value (for polling)
    double getY() const;

    // returns current (z) value (for polling)
    double getZ() const;

    // register a callback to be invoked on subsequent updates
    void addCallback( const MoAccelCallback & callback, void * data );

    // unregister a callback
    void removeCallback( const MoAccelCallback & callback );

public: // static members
    static MoAccel * instance();
    static MoAccel * ourInstance;
    static const float MAX_ACCEL_RATE;

private:
    MoAccel();
    ~MoAccel();

    // current values;
    double m_x;
    double m_y;
    double m_z;

    // update interval
    double m_updateInterval;

    // accelerometer Delegate
    AccelDelegate * accelDelegate;

    // queue of callbacks
    std::vector<MoAccelCallback> m_clients;
    std::vector<void *> m_clientData; // TODO: update the add and remove functions with the void * data

    // internal setup of the accelerometer
    void setup();		
};


#endif
